/*
   IM Caf - A Java Based Instant Messaging System
   Author: Bruno Freitas - 08/2004
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2004 Bruno Freitas
 
   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in
   the Software without restriction, including without limitation the rights to
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
   of the Software, and to permit persons to whom the Software is furnished to do
   so, subject to the following conditions:
 
   The above copyright notice and this permission notice shall be included in all
   copies or substantial portions of the Software.
 
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

import java.net.*;
import java.io.*;
import java.util.*;

public class DataHandler extends Thread {
   private ConnectionItem srcConn;
   private DBInterface dbInterface;
   private ConnectionsList connList;
   private CommData commData;
   
   public DataHandler(ConnectionItem srcConn, DBInterface dbInterface, ConnectionsList connList, CommData commData) {
      this.srcConn = srcConn;
      this.dbInterface = dbInterface;
      this.connList = connList;
      this.commData = commData;
   }
   
   public void run() {
      switch(commData.getCmd()) {
         case CommData.CMD_REGISTER_USER:
            cmdRegisterUser();
            break;
            
         case CommData.CMD_LOGIN_REQUEST:
            cmdLoginRequest();
            break;
            
         case CommData.CMD_INSTANT_MESSAGE:
            cmdInstantMessage();
            break;
            
         case CommData.CMD_ADD_CONTACT_REQ:
            cmdAddContactReq();
            break;
            
         case CommData.CMD_REQUEST_USER_STATUS:
            cmdRequestUserStatus();
            break;
            
         case CommData.CMD_REMOVE_CONTACT_REQ:
            cmdRemoveContactReq();
            break;
      }
   }
   
   private void cmdRegisterUser() {
      StringTokenizer tokens = new StringTokenizer(commData.getData(), ":");
      CommData answer;
      int ID = -1;
      
      answer = new CommData();
      
      try {
         ID = dbInterface.addUser(tokens.nextToken(), tokens.nextToken(), tokens.nextToken(), tokens.nextToken(), tokens.nextToken());
         answer.setCmd(CommData.CMD_GENERIC_MESSAGE);
         answer.setData("User registration was done! The new user ID is: " + ID);
      } catch (Exception e) {
         answer.setCmd(CommData.CMD_GENERIC_MESSAGE);
         answer.setData("Could not register new user!");
      }
      
      try  {
         srcConn.getWriter().writeObject(answer);
         srcConn.getWriter().flush();
      } catch (Exception e) {
         throw new RuntimeException(e);
      }
   }
   
   private void cmdLoginRequest() {
      StringTokenizer tokens = new StringTokenizer(commData.getData(), ":");
      int id = Integer.parseInt(tokens.nextToken());
      String pass = tokens.nextToken();
      boolean pass_ok;
      int session_id;
      String contactList;
      
      // Check the user's password
      try {
         pass_ok = (dbInterface.getUserPassword(id).compareTo(pass) == 0);
      } catch (Exception e) {
         pass_ok = false;
      }
      
      CommData answer = new CommData();
      
      if (pass_ok) {
         try {
            session_id = dbInterface.setUserOnLine(id, srcConn.getSocket().getInetAddress().getHostAddress().trim(), srcConn.getSocket().getPort());
         } catch (Exception e) {
            answer.setCmd(CommData.CMD_GENERIC_MESSAGE);
            answer.setData("Your login request was denied due to an error in the server");
            try {
               srcConn.getWriter().writeObject(answer);
               srcConn.getWriter().flush();
            } catch (Exception f) {
               throw new RuntimeException(f);
            }
            return;
         }
         
         try {
            contactList = dbInterface.getContactList(id, false);
         } catch (Exception e) {
            contactList = "";
         }
         
         answer.setCmd(CommData.CMD_LOGIN_GRANTED);
         answer.setTo(id);
         answer.setSessionID(session_id);
         answer.setData(contactList);
         
         try {
            srcConn.getWriter().writeObject(answer);
            srcConn.getWriter().flush();
         } catch (Exception e) {
            throw new RuntimeException(e);
         }
         
         // Notify every user who has this user in contact list about his new
         // state (Online)
         String ownersList;
         CommData notification = new CommData();
         
         try {
            ownersList = dbInterface.getOwnersList(id, true);
         } catch (Exception e) {
            ownersList = "";
         }
         
         StringTokenizer owners = new StringTokenizer(ownersList, ",");
         int ownerid;
         
         while (owners.hasMoreTokens()) {
            StringTokenizer owner = new StringTokenizer(owners.nextToken(), ":");
            
            ownerid = Integer.parseInt(owner.nextToken());
            
            notification.setCmd(CommData.CMD_USER_IS_ONLINE);
            notification.setData(String.valueOf(id));
            
            try {
               connList.getWriter(dbInterface.getUserIP(ownerid), dbInterface.getUserPort(ownerid)).writeObject(notification);
               connList.getWriter(dbInterface.getUserIP(ownerid), dbInterface.getUserPort(ownerid)).flush();
            } catch (Exception e) {
               //continue;
            }
         }
      } else {
         answer.setCmd(CommData.CMD_GENERIC_MESSAGE);
         answer.setData("Your login request was denied!");
         
         try {
            srcConn.getWriter().writeObject(answer);
            srcConn.getWriter().flush();
         } catch (Exception f) {
            throw new RuntimeException(f);
         }
      }
   }
   
   private void cmdInstantMessage() {
      CommData answer;
      boolean isDestOnline, destHasSender;
      
      try {
         isDestOnline = dbInterface.isUserOnLine(commData.getTo());
      } catch (Exception e) {
         isDestOnline = false;
      }
      
      if (!isDestOnline) {
         answer = new CommData();
         answer.setCmd(CommData.CMD_GENERIC_MESSAGE);
         answer.setData("The message could not be delivered. Maybe contact is offline");
         
         try {
            srcConn.getWriter().writeObject(answer);
            srcConn.getWriter().flush();
         } catch (Exception f) {
            throw new RuntimeException(f);
         }
         
         return;
      }
      
      // Check if the sender is in destination's contact list. If not, then add it first!
      destHasSender = false;
      
      try {
         destHasSender = dbInterface.ownerHasContact(commData.getTo(), commData.getFrom());
      } catch (Exception e) {
         throw new RuntimeException(e);
      }
      
      if (!destHasSender) {
         answer = new CommData();
         
         try {
            dbInterface.addContact(commData.getTo(), commData.getFrom());
            answer.setCmd(CommData.CMD_ADD_CONTACT_OK);
            answer.setData(commData.getFrom() + ":" + dbInterface.getUserNick(commData.getFrom()) + ":" + ((dbInterface.isUserOnLine(commData.getFrom()))?1:0));
         } catch (Exception e) {
            throw new RuntimeException(e);
         }
         
         try {
            connList.getWriter(dbInterface.getUserIP(commData.getTo()), dbInterface.getUserPort(commData.getTo())).writeObject(answer);
            connList.getWriter(dbInterface.getUserIP(commData.getTo()), dbInterface.getUserPort(commData.getTo())).flush();
         } catch (Exception e) {
            //throw new RuntimeException(e);
         }
         
      }
      
      // Finally, deliver the instant message!
      try {
         connList.getWriter(dbInterface.getUserIP(commData.getTo()), dbInterface.getUserPort(commData.getTo())).writeObject(commData);
         connList.getWriter(dbInterface.getUserIP(commData.getTo()), dbInterface.getUserPort(commData.getTo())).flush();
      } catch (Exception e) {
         //throw new RuntimeException(e);
         // Silent mode!! :-)
      }
   }
   
   private void cmdAddContactReq() {
      CommData answer = new CommData();
      
      try {
         dbInterface.addContact(commData.getFrom(), commData.getTo());
         answer.setCmd(CommData.CMD_ADD_CONTACT_OK);
         answer.setData(commData.getTo() + ":" + dbInterface.getUserNick(commData.getTo()) + ":" + ((dbInterface.isUserOnLine(commData.getTo()))?1:0));
      } catch (Exception e) {
         answer.setCmd(CommData.CMD_GENERIC_MESSAGE);
         answer.setData("The contact could not be added to your contact list.");
      }
      
      try {
         srcConn.getWriter().writeObject(answer);
         srcConn.getWriter().flush();
      } catch (Exception f) {
         throw new RuntimeException(f);
      }
   }
   
   private void cmdRequestUserStatus() {
      CommData answer = new CommData();
      
      answer.setCmd(CommData.CMD_USER_STATUS_REPLY);
      
      try {
         answer.setData(commData.getTo() + ":" + ((dbInterface.isUserOnLine(commData.getTo()))?1:0));
      } catch (Exception e) {
         answer.setData(commData.getTo() + ":0");
      }
      
      try {
         srcConn.getWriter().writeObject(answer);
         srcConn.getWriter().flush();
      } catch (Exception f) {
         throw new RuntimeException(f);
      }
   }
   
   private void cmdRemoveContactReq() {
      CommData answer = new CommData();
      
      try {
         dbInterface.removeContact(commData.getFrom(), commData.getTo());
         answer.setCmd(CommData.CMD_REMOVE_CONTACT_OK);
         answer.setTo(commData.getTo());
      } catch (Exception e) {
         answer.setCmd(CommData.CMD_GENERIC_MESSAGE);
         answer.setData("The contact could not be removed from your contact list.");
      }
      
      try {
         srcConn.getWriter().writeObject(answer);
         srcConn.getWriter().flush();
      } catch (Exception f) {
         throw new RuntimeException(f);
      }
      
   }
}
